# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 10.0.0


from . import AWSObject
from . import AWSProperty
from troposphere import Tags
from .validators import boolean


class Filter(AWSProperty):
    props = {
        'Contains': ([str], False),
        'Eq': ([str], False),
        'Exists': (boolean, False),
        'Neq': ([str], False),
        'Property': (str, True),
    }


class ArchiveRule(AWSProperty):
    props = {
        'Filter': ([Filter], True),
        'RuleName': (str, True),
    }


class Analyzer(AWSObject):
    resource_type = "AWS::AccessAnalyzer::Analyzer"

    props = {
        'AnalyzerName': (str, False),
        'ArchiveRules': ([ArchiveRule], False),
        'Tags': (Tags, False),
        'Type': (str, True),
    }
