# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 3.3.0


from . import AWSObject
from . import AWSProperty
from troposphere import Tags


class EndpointDetails(AWSProperty):
    props = {
        'VpcEndpointId': (str, True),
    }


class IdentityProviderDetails(AWSProperty):
    props = {
        'InvocationRole': (str, True),
        'Url': (str, True),
    }


class Server(AWSObject):
    resource_type = "AWS::Transfer::Server"

    props = {
        'EndpointDetails': (EndpointDetails, False),
        'EndpointType': (str, False),
        'IdentityProviderDetails': (IdentityProviderDetails, False),
        'IdentityProviderType': (str, False),
        'LoggingRole': (str, False),
        'Tags': (Tags, False),
    }


class User(AWSObject):
    resource_type = "AWS::Transfer::User"

    props = {
        'HomeDirectory': (str, False),
        'Policy': (str, False),
        'Role': (str, True),
        'ServerId': (str, True),
        'SshPublicKeys': ([str], False),
        'Tags': (Tags, False),
        'UserName': (str, True),
    }
