# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 10.0.0


from . import AWSObject
from troposphere import Tags


class Discoverer(AWSObject):
    resource_type = "AWS::EventSchemas::Discoverer"

    props = {
        'Description': (str, False),
        'SourceArn': (str, True),
        'Tags': (Tags, False),
    }


class Registry(AWSObject):
    resource_type = "AWS::EventSchemas::Registry"

    props = {
        'Description': (str, False),
        'RegistryName': (str, False),
        'Tags': (Tags, False),
    }


class Schema(AWSObject):
    resource_type = "AWS::EventSchemas::Schema"

    props = {
        'Content': (str, True),
        'Description': (str, False),
        'RegistryName': (str, True),
        'SchemaName': (str, False),
        'Tags': (Tags, False),
        'Type': (str, True),
    }
